﻿using System;
using System.IO;
using System.Threading;

using Google.Apis.Auth.OAuth2;
using Google.Apis.Services;
using Google.Apis.Util.Store;
using Google.Apis.YouTube.v3;
using Google.Apis.YouTube.v3.Data;

namespace TestProject
{
    /// <summary>
    /// 프로그램
    /// </summary>
    class Program
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region 프로그램 시작하기 - Main()

        /// <summary>
        /// 프로그램 시작하기
        /// </summary>
        private static void Main()
        {
            #region 구글 OAuth2 인증을 한다.

            string clientSecretsFilePath = @"E:\client_secret.json"; // 클라이언트 시크릿 파일 경로를 설정한다.

            UserCredential credential;

            using(FileStream stream = new FileStream(clientSecretsFilePath, FileMode.Open, FileAccess.Read))
            {
                credential = GoogleWebAuthorizationBroker.AuthorizeAsync
                (
                    GoogleClientSecrets.FromStream(stream).Secrets,
                    new[] { YouTubeService.Scope.Youtube },
                    "user",
                    CancellationToken.None,
                    new FileDataStore("TestProject")
                ).GetAwaiter().GetResult();
            }

            #endregion
            #region 유튜브 서비스를 설정한다.

            YouTubeService service = new YouTubeService
            (
                new BaseClientService.Initializer()
                {
                    HttpClientInitializer = credential,
                    ApplicationName       = "TestProject"
                }
            );

            #endregion
            #region 채널 요청을 설정한다.

            ChannelsResource.ListRequest channelRequest = service.Channels.List("ContentDetails");

            channelRequest.Mine = true;

            #endregion

            ChannelListResponse channelResponse = channelRequest.Execute();

            int i = 0;

            foreach(Channel channel in channelResponse.Items)
            {
                string playListID = channel.ContentDetails.RelatedPlaylists.Uploads;

                string nextPageToken = string.Empty;

                do
                {
                    #region 재생 목록 요청을 설정한다.

                    PlaylistItemsResource.ListRequest playlistRequest = service.PlaylistItems.List("Snippet");

                    playlistRequest.PlaylistId = playListID;
                    playlistRequest.MaxResults = 50;
                    playlistRequest.PageToken  = nextPageToken;

                    #endregion

                    PlaylistItemListResponse playlistResponse = playlistRequest.Execute();

                    foreach(PlaylistItem item in playlistResponse.Items)
                    {
                        Console.WriteLine(++i);
                        Console.WriteLine($"비디오 ID : {item.Snippet.ResourceId.VideoId}");
                        Console.WriteLine($"제목      : {item.Snippet.Title}");
                        Console.WriteLine();
                    }

                    nextPageToken = playlistResponse.NextPageToken;
                }
                while(nextPageToken != null);
            }
        }

        #endregion
    }
}